@@ -20,9 +20,9 @@
 """Provides getCharacterName that maps punctuation marks and other
 individual characters into localized words."""
 
-__id__        = "$Id: chnames.py 2433 2007-05-29 15:08:39Z richb $"
-__version__   = "$Revision: 2433 $"
-__date__      = "$Date: 2007-05-29 11:08:39 -0400 (Tue, 29 May 2007) $"
+__id__        = "$Id: chnames.py 2799 2007-09-07 13:43:38Z wwalker $"
+__version__   = "$Revision: 2799 $"
+__date__      = "$Date: 2007-09-07 09:43:38 -0400 (Fri, 07 Sep 2007) $"
 __copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -275,7 +275,7 @@ chnames[u'\u00b9'] = _("superscript 1")
 
 # Translators: this is the spoken word for the character 'º' (U+00ba)
 #
-chnames[u'\u00ba'] = _("degrees")
+chnames[u'\u00ba'] = _("ordinal")
 
 # Translators: this is the spoken word for the character '»' (U+00bb)
 #
