@@ -19,9 +19,9 @@
 
 """The main module for the Orca screen reader."""
 
-__id__        = "$Id: orca.py 2953 2007-10-01 20:02:55Z wwalker $"
-__version__   = "$Revision: 2953 $"
-__date__      = "$Date: 2007-10-01 16:02:55 -0400 (Mon, 01 Oct 2007) $"
+__id__        = "$Id: orca.py 3227 2007-11-08 21:28:31Z eitani $"
+__version__   = "$Revision: 3227 $"
+__date__      = "$Date: 2007-11-08 16:28:31 -0500 (Thu, 08 Nov 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -45,12 +45,13 @@ except:
     pass
 
 import getopt
+import gnome
 import os
 import signal
 import time
 import unicodedata
 
-import atspi
+import pyatspi
 import braille
 import debug
 import httpserver
@@ -59,16 +60,16 @@ import keybindings
 import mag
 import orca_state
 import platform
-import rolenames
 import settings
 import speech
 
 from input_event import BrailleEvent
 from input_event import KeyboardEvent
 from input_event import MouseButtonEvent
-from input_event import InputEventHandler
+from input_event import keyEventToString
 
 from orca_i18n import _           # for gettext support
+from orca_i18n import Q_          # to provide qualified translatable strings
 
 if settings.debugMemoryUsage:
     import gc
@@ -160,25 +161,34 @@ def setLocusOfFocus(event, obj, notifyPresentationManager=True):
         return
 
     oldLocusOfFocus = orca_state.locusOfFocus
-    if oldLocusOfFocus and not oldLocusOfFocus.valid:
+    try:
+        # Just to see if we have a valid object.
+        oldLocusOfFocus.getRole()
+    except:
+        # Either it's None or it's an invalid remote object.
         oldLocusOfFocus = None
 
     orca_state.locusOfFocus = obj
-    if orca_state.locusOfFocus and not orca_state.locusOfFocus.valid:
+    try:
+        app = orca_state.locusOfFocus.getApplication()
+    except:
         orca_state.locusOfFocus = None
-
-    if orca_state.locusOfFocus:
-        appname = ""
-        if not orca_state.locusOfFocus.app:
+        if event:
+            debug.println(debug.LEVEL_FINE,
+                          "LOCUS OF FOCUS: None event='%s'" % event.type)
+        else:
+            debug.println(debug.LEVEL_FINE,
+                          "LOCUS OF FOCUS: None event=None")
+    else:
+        if not app:
             appname = "None"
         else:
-            appname = "'" + orca_state.locusOfFocus.app.name + "'"
-
+            appname = "'" + app.name + "'"
         debug.println(debug.LEVEL_FINE,
                       "LOCUS OF FOCUS: app=%s name='%s' role='%s'" \
                       % (appname,
                          orca_state.locusOfFocus.name,
-                         orca_state.locusOfFocus.role))
+                         orca_state.locusOfFocus.getRoleName()))
 
         if event:
             debug.println(debug.LEVEL_FINE,
@@ -186,13 +196,6 @@ def setLocusOfFocus(event, obj, notifyPresentationManager=True):
         else:
             debug.println(debug.LEVEL_FINE,
                           "                event=None")
-    else:
-        if event:
-            debug.println(debug.LEVEL_FINE,
-                          "LOCUS OF FOCUS: None event='%s'" % event.type)
-        else:
-            debug.println(debug.LEVEL_FINE,
-                          "LOCUS OF FOCUS: None event=None")
 
     if notifyPresentationManager and _currentPresentationManager >= 0:
         _PRESENTATION_MANAGERS[_currentPresentationManager].\
@@ -226,13 +229,13 @@ def _onChildrenChanged(e):
     - e: at-spi event from the at-api registry
     """
 
-    registry = atspi.Registry()
-    if e.source == registry.desktop:
+    desktop = pyatspi.Registry.getDesktop(0)
+    if e.source == desktop:
 
         # If the desktop is empty, the user has logged out-- shutdown Orca
         #
         try:
-            if registry.desktop.childCount == 0:
+            if desktop.childCount == 0:
                 speech.speak(_("Goodbye."))
                 shutdown()
                 return
@@ -248,113 +251,17 @@ def _onMouseButton(e):
     - e: at-spi event from the at-api registry
     """
 
-    event = atspi.Event(e)
-    orca_state.lastInputEvent = MouseButtonEvent(event)
+    mouse_event = MouseButtonEvent(e)
+    orca_state.lastInputEvent = mouse_event
 
     # A mouse button event looks like: mouse:button:1p, where the
     # number is the button number and the 'p' is either 'p' or 'r',
     # meaning pressed or released.  We only want to stop speech on
     # button presses.
     #
-    if event.type.endswith("p"):
+    if mouse_event.pressed:
         speech.stop()
 
-########################################################################
-#                                                                      #
-# Keyboard Event Recording Support                                     #
-#                                                                      #
-########################################################################
-
-_recordingKeystrokes = False
-_keystrokesFile = None
-
-def _closeKeystrokeWindowAndRecord(entry, window):
-    global _keystrokesFile
-    window.destroy()
-    entry_text = entry.get_text()
-    _keystrokesFile = open(entry_text, 'w')
-
-def _closeKeystrokeWindowAndCancel(window):
-    global _recordingKeystrokes
-    window.destroy()
-    _recordingKeystrokes = False
-
-def toggleKeystrokeRecording(script=None, inputEvent=None):
-    """Toggles the recording of keystrokes on and off.  When the
-    user presses the magic key (Pause), Orca will pop up a window
-    requesting a filename.  When the user presses the close button,
-    Orca will start recording keystrokes to the file and will continue
-    recording them until the user presses the magic key again.
-
-    This functionality is used primarily to help gather keystroke
-    information for regression testing purposes.  The keystrokes are
-    recorded in such a way that they can be played back via the
-    src/tools/play_keystrokes.py utility.
-
-    Arguments:
-    - inputEvent: the key event (if any) which caused this to be called.
-
-    Returns True indicating the event should be consumed.
-    """
-
-    global _recordingKeystrokes
-    global _keystrokesFile
-
-    if _recordingKeystrokes:
-        # If the filename entry window is still up, we don't have a file
-        # yet.
-        #
-        if _keystrokesFile:
-            _keystrokesFile.close()
-            _keystrokesFile = None
-            _recordingKeystrokes = False
-    else:
-        _recordingKeystrokes = True
-        window = gtk.Window(gtk.WINDOW_TOPLEVEL)
-        window.set_title("Keystroke Filename")
-
-        vbox = gtk.VBox(False, 0)
-        window.add(vbox)
-        vbox.show()
-
-        entry = gtk.Entry()
-        entry.set_max_length(50)
-        entry.set_editable(True)
-        entry.set_text("keystrokes.txt")
-        entry.select_region(0, len(entry.get_text()))
-        # For now, do not allow "Return" to close the window - the reason
-        # for this is that the key press closes the window, and the key
-        # release will end up getting recorded.
-        #
-        #entry.connect("activate", _closeKeystrokeWindow, window)
-        vbox.pack_start(entry, True, True, 0)
-        entry.show()
-
-        hbox = gtk.HBox(False, 0)
-        vbox.add(hbox)
-        hbox.show()
-
-        ok = gtk.Button(stock=gtk.STOCK_OK)
-        ok.connect("clicked", lambda w: _closeKeystrokeWindowAndRecord(\
-            entry, \
-            window))
-
-        cancel = gtk.Button(stock=gtk.STOCK_CANCEL)
-        cancel.connect("clicked", lambda w: _closeKeystrokeWindowAndCancel(\
-            window))
-
-        vbox.pack_start(cancel, True, True, 0)
-        vbox.pack_start(ok, True, True, 0)
-
-        ok.set_flags(gtk.CAN_DEFAULT)
-        ok.grab_default()
-        ok.show()
-        cancel.show()
-
-        window.set_modal(True)
-        window.show()
-    return True
-
 ########################################################################
 #                                                                      #
 # DEBUG support.                                                       #
@@ -362,49 +269,27 @@ def toggleKeystrokeRecording(script=None, inputEvent=None):
 ########################################################################
 
 def cycleDebugLevel(script=None, inputEvent=None):
-    global _debugLevel
+    levels = [debug.LEVEL_ALL, "all",
+              debug.LEVEL_FINEST, "finest",
+              debug.LEVEL_FINER, "finer",
+              debug.LEVEL_FINE, "fine",
+              debug.LEVEL_CONFIGURATION, "configuration",
+              debug.LEVEL_INFO, "info",
+              debug.LEVEL_WARNING, "warning",
+              debug.LEVEL_SEVERE, "severe",
+              debug.LEVEL_OFF, "off"]
 
-    level = debug.debugLevel
+    try:
+        levelIndex = levels.index(debug.debugLevel) + 2
+    except:
+        levelIndex = 0
+    else:
+        if levelIndex >= len(levels):
+            levelIndex = 0
 
-    if level == debug.LEVEL_ALL:
-        level = debug.LEVEL_FINEST
-    elif level == debug.LEVEL_FINEST:
-        level = debug.LEVEL_FINER
-    elif level == debug.LEVEL_FINER:
-        level = debug.LEVEL_FINE
-    elif level == debug.LEVEL_FINE:
-        level = debug.LEVEL_CONFIGURATION
-    elif level == debug.LEVEL_CONFIGURATION:
-        level = debug.LEVEL_INFO
-    elif level == debug.LEVEL_INFO:
-        level = debug.LEVEL_WARNING
-    elif level == debug.LEVEL_WARNING:
-        level = debug.LEVEL_SEVERE
-    elif level == debug.LEVEL_SEVERE:
-        level = debug.LEVEL_OFF
-    elif level == debug.LEVEL_OFF:
-        level = debug.LEVEL_ALL
+    debug.debugLevel = levels[levelIndex]
 
-    debug.debugLevel = level
-
-    if level == debug.LEVEL_ALL:
-        speech.speak("Debug level all.")
-    elif level == debug.LEVEL_FINEST:
-        speech.speak("Debug level finest.")
-    elif level == debug.LEVEL_FINER:
-        speech.speak("Debug level finer.")
-    elif level == debug.LEVEL_FINE:
-        speech.speak("Debug level fine.")
-    elif level == debug.LEVEL_CONFIGURATION:
-        speech.speak("Debug level configuration.")
-    elif level == debug.LEVEL_INFO:
-        speech.speak("Debug level info.")
-    elif level == debug.LEVEL_WARNING:
-        speech.speak("Debug level warning.")
-    elif level == debug.LEVEL_SEVERE:
-        speech.speak("Debug level severe.")
-    elif level == debug.LEVEL_OFF:
-        speech.speak("Debug level off.")
+    speech.speak("Debug level %s." % levels[levelIndex + 1])
 
     return True
 
@@ -445,7 +330,7 @@ def _isPrintableKey(event_string):
                 and (unicodeString.isalnum() or unicodeString.isspace()
                 or unicodedata.category(unicodeString)[0] in ('P', 'S'))
     debug.println(debug.LEVEL_FINEST,
-                  "orca._echoPrintableKey: returning: %s" % reply)
+                  "orca._isPrintableKey: returning: %s" % reply)
     return reply
 
 def _isModifierKey(event_string):
@@ -457,16 +342,13 @@ def _isModifierKey(event_string):
     Returns True if this is a modifier key
     """
 
-    # [[[TODO:richb - the Fn key on my laptop doesn't seem to generate an
-    #    event.]]]
-
     modifierKeys = [ 'Alt_L', 'Alt_R', 'Control_L', 'Control_R', \
                      'Shift_L', 'Shift_R', 'Meta_L', 'Meta_R' ]
     modifierKeys.extend(settings.orcaModifierKeys)
 
     reply = event_string in modifierKeys
     debug.println(debug.LEVEL_FINEST,
-                  "orca._echoModifierKey: returning: %s" % reply)
+                  "orca._isModifierKey: returning: %s" % reply)
     return reply
 
 def _isLockingKey(event_string):
@@ -483,7 +365,7 @@ def _isLockingKey(event_string):
     reply = event_string in lockingKeys \
             and not event_string in settings.orcaModifierKeys
     debug.println(debug.LEVEL_FINEST,
-                  "orca._echoLockingKey: returning: %s" % reply)
+                  "orca._isLockingKey: returning: %s" % reply)
     return reply
 
 def _isFunctionKey(event_string):
@@ -504,7 +386,7 @@ def _isFunctionKey(event_string):
 
     reply = event_string in functionKeys
     debug.println(debug.LEVEL_FINEST,
-                  "orca._echoFunctionKey: returning: %s" % reply)
+                  "orca._isFunctionKey: returning: %s" % reply)
     return reply
 
 def _isActionKey(event_string):
@@ -521,7 +403,28 @@ def _isActionKey(event_string):
 
     reply = event_string in actionKeys
     debug.println(debug.LEVEL_FINEST,
-                  "orca._echoActionKey: returning: %s" % reply)
+                  "orca._isActionKey: returning: %s" % reply)
+    return reply
+
+def _isNavigationKey(event_string):
+    """Return an indication of whether this is a navigation (arrow) key
+    or if the user has the Orca modifier key held done.
+
+    Arguments:
+    - event: the event string
+    - modifiers: key modifiers state
+
+    Returns True if this is a navigation key.
+    """
+
+    global _orcaModifierPressed
+
+    navigationKeys = [ "Left", "Right", "Up", "Down" ]
+
+    reply = (event_string in navigationKeys) or _orcaModifierPressed
+
+    debug.println(debug.LEVEL_FINEST,
+                  "orca._isNavigationKey: returning: %s" % reply)
     return reply
 
 class KeyEventType:
@@ -548,6 +451,9 @@ class KeyEventType:
     """An action key event."""
     ACTION = 'ACTION'
 
+    """A navigation key event."""
+    NAVIGATION = 'NAVIGATION'
+
 def _keyEcho(event):
     """If the keyEcho setting is enabled, check to see what type of key
     event it is and echo it via speech, if the user wants that type of
@@ -564,7 +470,7 @@ def _keyEcho(event):
     # field, then don't echo it.
     #
     if orca_state.locusOfFocus \
-        and (orca_state.locusOfFocus.role == rolenames.ROLE_PASSWORD_TEXT):
+        and (orca_state.locusOfFocus.getRole() == pyatspi.ROLE_PASSWORD_TEXT):
         return
 
     event_string = event.event_string
@@ -577,28 +483,33 @@ def _keyEcho(event):
     #
     if settings.enableKeyEcho:
 
-        if _isPrintableKey(event_string):
-            if not settings.enablePrintableKeys:
-                return
-            type = KeyEventType.PRINTABLE
-
-        elif _isModifierKey(event_string):
+        if _isModifierKey(event_string):
             if not settings.enableModifierKeys:
                 return
-            type = KeyEventType.MODIFIER
+            eventType = KeyEventType.MODIFIER
+
+        elif _isNavigationKey(event_string):
+            if not settings.enableNavigationKeys:
+                return
+            eventType = KeyEventType.NAVIGATION
+
+        elif _isPrintableKey(event_string):
+            if not settings.enablePrintableKeys:
+                return
+            eventType = KeyEventType.PRINTABLE
 
         elif _isLockingKey(event_string):
             if not settings.enableLockingKeys:
                 return
-            type = KeyEventType.LOCKING
+            eventType = KeyEventType.LOCKING
 
             modifiers = event.modifiers
 
             if event_string == "Caps_Lock":
-                if modifiers & (1 << atspi.Accessibility.MODIFIER_SHIFTLOCK):
-                    type = KeyEventType.LOCKING_UNLOCKED
+                if modifiers & (1 << pyatspi.MODIFIER_SHIFTLOCK):
+                    eventType = KeyEventType.LOCKING_UNLOCKED
                 else:
-                    type = KeyEventType.LOCKING_LOCKED
+                    eventType = KeyEventType.LOCKING_LOCKED
 
             elif event_string == "Num_Lock":
                 # [[[TODO: richb - we are not getting a correct modifier
@@ -606,21 +517,21 @@ def _keyEcho(event):
                 # Commenting out the speaking of the bogus on/off state
                 # until this can be fixed.]]]
                 #
-                #if modifiers & (1 << atspi.Accessibility.MODIFIER_NUMLOCK):
-                #    type = KeyEventType.LOCKING_UNLOCKED
+                #if modifiers & (1 << pyatspi.MODIFIER_NUMLOCK):
+                #    eventType = KeyEventType.LOCKING_UNLOCKED
                 #else:
-                #    type = KeyEventType.LOCKING_LOCKED
+                #    eventType = KeyEventType.LOCKING_LOCKED
                 pass
 
         elif _isFunctionKey(event_string):
             if not settings.enableFunctionKeys:
                 return
-            type = KeyEventType.FUNCTION
+            eventType = KeyEventType.FUNCTION
 
         elif _isActionKey(event_string):
             if not settings.enableActionKeys:
                 return
-            type = KeyEventType.ACTION
+            eventType = KeyEventType.ACTION
 
         else:
             debug.println(debug.LEVEL_FINEST,
@@ -635,7 +546,7 @@ def _keyEcho(event):
         #
         orca_state.lastKeyEchoTime = time.time()
 
-        speech.speakKeyEvent(event_string, type)
+        speech.speakKeyEvent(event_string, eventType)
 
 def _processKeyCaptured(event):
     """Called when a new key event arrives and orca_state.capturingKeys=True.
@@ -644,20 +555,36 @@ def _processKeyCaptured(event):
 
     if event.type == 0:
         if _isModifierKey(event.event_string) \
-               or event.event_string == "Return":
-            pass
-        elif event.event_string == "Escape":
-            orca_state.capturingKeys = False
+           or _isLockingKey(event.event_string):
+            return True
         else:
-            # Translators: this is a spoken prompt letting the user know
-            # Orca has recorded a new key combination (e.g., Alt+Ctrl+g)
-            # based upon their input.
+            # We want to capture this event, after first doing a little
+            # clean up.  Eliminate modifiers we're not interested in.
             #
-            speech.speak(_("Key captured: %s. Press enter to confirm.") \
-                         % str(event.event_string))
+            mask = (1 << settings.MODIFIER_ORCA |
+                    1 << pyatspi.MODIFIER_ALT |
+                    1 << pyatspi.MODIFIER_SHIFT |
+                    1 << pyatspi.MODIFIER_CONTROL |
+                    1 << pyatspi.MODIFIER_META2 |
+                    1 << pyatspi.MODIFIER_META3)
+            event.modifiers = event.modifiers & mask
+
+            # We want the keyname rather than the printable character.
+            # If it's not on the keypad, get the name of the unshifted
+            # character. (i.e. "1" instead of "!")
+            #
+            keymap = gtk.gdk.keymap_get_default()
+            entries = keymap.get_entries_for_keycode(event.hw_code)
+            event.event_string = gtk.gdk.keyval_name(entries[0][0])
+            if event.event_string.startswith("KP"):
+                name = gtk.gdk.keyval_name(entries[1][0])
+                if name.startswith("KP"):
+                    event.event_string = name
+
             orca_state.lastCapturedKey = event
     else:
         pass
+
     return False
 
 def _processKeyboardEvent(event):
@@ -683,21 +610,17 @@ def _processKeyboardEvent(event):
     # however, will translate the event_string for control
     # characters to their upper case ASCII equivalent.
     #
-    string = atspi.KeystrokeListener.keyEventToString(event)
-    if _recordingKeystrokes and _keystrokesFile \
-       and (event.event_string != "Pause") \
-       and (event.event_string != "F21"):
-        _keystrokesFile.write(string + "\n")
+    string = keyEventToString(event)
     debug.printInputEvent(debug.LEVEL_FINE, string)
 
     keyboardEvent = KeyboardEvent(event)
 
     # See if this is one of our special Orca modifier keys.
     #
-    # [[[TODO: WDW - Note that just looking at the keycode should
-    # suffice, but there is a "feature" in the Java Access Bridge
-    # where it chooses to emit Java platform-independent keycodes
-    # instead of the keycodes for the base platform:
+    # Just looking at the keycode should suffice, but there is a 
+    # "feature" in the Java Access Bridge where it chooses to emit 
+    # Java platform-independent keycodes instead of the keycodes 
+    # for the base platform:
     #
     # http://bugzilla.gnome.org/show_bug.cgi?id=106004
     # http://bugzilla.gnome.org/show_bug.cgi?id=318615
@@ -715,7 +638,7 @@ def _processKeyboardEvent(event):
     # the following expression is False, only the \ will be viewed as
     # the Orca modifier (i.e., Shift+\ will still function as the |
     # character).  In general, I think we want to avoid sucking in all
-    # possible keysyms because it can have unexpected results.]]]
+    # possible keysyms because it can have unexpected results.
     #
     if False:
         allPossibleKeysyms = []
@@ -726,24 +649,24 @@ def _processKeyboardEvent(event):
 
     isOrcaModifier = allPossibleKeysyms.count(keyboardEvent.event_string) > 0
 
-    if event.type == atspi.Accessibility.KEY_PRESSED_EVENT:
+    if event.type == pyatspi.KEY_PRESSED_EVENT:
         # Key presses always interrupt speech.
         #
         speech.stop()
 
-        # If learn mode is enabled, it will echo the keys.
-        #
-        if not settings.learnModeEnabled:
-            _keyEcho(keyboardEvent)
-
         # We treat the Insert key as a modifier - so just swallow it and
         # set our internal state.
         #
         if isOrcaModifier:
             _orcaModifierPressed = True
 
+        # If learn mode is enabled, it will echo the keys.
+        #
+        if not settings.learnModeEnabled:
+            _keyEcho(keyboardEvent)
+
     elif isOrcaModifier \
-        and (keyboardEvent.type == atspi.Accessibility.KEY_RELEASED_EVENT):
+        and (keyboardEvent.type == pyatspi.KEY_RELEASED_EVENT):
         _orcaModifierPressed = False
 
     if _orcaModifierPressed:
@@ -758,7 +681,7 @@ def _processKeyboardEvent(event):
     consumed = False
     try:
         if orca_state.capturingKeys:
-            _processKeyCaptured(keyboardEvent)
+            consumed = _processKeyCaptured(keyboardEvent)
         else:
             consumed = _keyBindings.consumeKeyboardEvent(None, keyboardEvent)
             if (not consumed) and (_currentPresentationManager >= 0):
@@ -766,7 +689,7 @@ def _processKeyboardEvent(event):
                            processKeyboardEvent(keyboardEvent)
             if (not consumed) and settings.learnModeEnabled:
                 if keyboardEvent.type \
-                    == atspi.Accessibility.KEY_PRESSED_EVENT:
+                    == pyatspi.KEY_PRESSED_EVENT:
                     clickCount = orca_state.activeScript.getClickCount(\
                                                 orca_state.lastInputEvent,
                                                 keyboardEvent)
@@ -812,8 +735,6 @@ def _processBrailleEvent(command):
     Returns True if the event was consumed; otherwise False
     """
 
-    # [[[TODO: WDW - probably should add braille bindings to this module.]]]
-
     consumed = False
 
     # Braille key presses always interrupt speech.
@@ -902,7 +823,7 @@ def loadUserSettings(script=None, inputEvent=None):
     # precedence over everything else.
     #
     for key in _commandLineSettings:
-        settings.__dict__[key] = _commandLineSettings[key]
+        setattr(settings, key, _commandLineSettings[key])
 
     if settings.enableSpeech:
         try:
@@ -994,7 +915,6 @@ def _showAppPreferencesGUI(script=None, inputEvent=None):
         module.showPreferencesUI()
     except:
         debug.printException(debug.LEVEL_SEVERE)
-        pass
 
     return True
 
@@ -1013,7 +933,6 @@ def _showPreferencesGUI(script=None, inputEvent=None):
         module.showPreferencesUI()
     except:
         debug.printException(debug.LEVEL_SEVERE)
-        pass
 
     return True
 
@@ -1034,7 +953,6 @@ def _showMainWindowGUI(script=None, inputEvent=None):
             module.hideMainUI()
     except:
         debug.printException(debug.LEVEL_SEVERE)
-        pass
 
     return True
 
@@ -1053,7 +971,19 @@ def _showPreferencesConsole(script=None, inputEvent=None):
         module.showPreferencesUI()
     except:
         debug.printException(debug.LEVEL_SEVERE)
-        pass
+
+    return True
+
+def helpForOrca(script=None, inputEvent=None):
+    """Show Orca Help window (part of the GNOME Access Guide).
+
+    Returns True to indicate the input event has been consumed.
+    """
+
+    props = { gnome.PARAM_APP_DATADIR : '/usr/share'}
+    prog = gnome.program_init('orca', '1.0', properties=props)
+    gnome.help_display_with_doc_id(prog, 'gnome-access-guide', 
+                                         'gnome-access-guide', 'ats-2')
 
     return True
 
@@ -1075,7 +1005,6 @@ def quitOrca(script=None, inputEvent=None):
             module.showQuitUI()
         except:
             debug.printException(debug.LEVEL_SEVERE)
-            pass
 
     return True
 
@@ -1093,7 +1022,6 @@ def _showFindGUI(script=None, inputEvent=None):
         module.showFindUI()
     except:
         debug.printException(debug.LEVEL_SEVERE)
-        pass
 
 # If True, this module has been initialized.
 #
@@ -1129,7 +1057,7 @@ def init(registry):
     # Create and load an app's script when it is added to the desktop
     #
     registry.registerEventListener(_onChildrenChanged,
-                                   "object:children-changed:")
+                                   "object:children-changed")
 
     # We also want to stop speech when a mouse button is pressed.
     #
@@ -1138,7 +1066,15 @@ def init(registry):
 
     loadUserSettings()
 
-    registry.registerKeystrokeListeners(_processKeyboardEvent)
+    masks = []
+    mask = 0
+    while mask <= (1 << pyatspi.MODIFIER_NUMLOCK):
+        masks.append(mask)
+        mask += 1
+    pyatspi.Registry.registerKeystrokeListener(
+        _processKeyboardEvent,
+        mask=masks,
+        kind=(pyatspi.KEY_PRESSED_EVENT, pyatspi.KEY_RELEASED_EVENT))
 
     if settings.timeoutCallback and (settings.timeoutTime > 0):
         signal.alarm(0)
@@ -1170,6 +1106,9 @@ def start(registry):
 
     if settings.timeoutCallback and (settings.timeoutTime > 0):
         signal.alarm(0)
+    
+    if settings.cacheValues:
+        pyatspi.setCacheLevel(pyatspi.CACHE_PROPERTIES)
 
     registry.start()
 
@@ -1208,11 +1147,10 @@ def shutdown(script=None, inputEvent=None):
 
     # Deregister our event listeners
     #
-    registry = atspi.Registry()
-    registry.deregisterEventListener(_onChildrenChanged,
-                                     "object:children-changed:")
-    registry.deregisterEventListener(_onMouseButton,
-                                     "mouse:button")
+    pyatspi.Registry.deregisterEventListener(_onChildrenChanged,
+                                             "object:children-changed")
+    pyatspi.Registry.deregisterEventListener(_onMouseButton,
+                                             "mouse:button")
 
     if _currentPresentationManager >= 0:
         _PRESENTATION_MANAGERS[_currentPresentationManager].deactivate()
@@ -1222,11 +1160,11 @@ def shutdown(script=None, inputEvent=None):
     if settings.enableSpeech:
         speech.shutdown()
     if settings.enableBraille:
-        braille.shutdown();
+        braille.shutdown()
     if settings.enableMagnifier:
-        mag.shutdown();
+        mag.shutdown()
 
-    registry.stop()
+    pyatspi.Registry.stop()
 
     if settings.timeoutCallback and (settings.timeoutTime > 0):
         signal.alarm(0)
@@ -1286,22 +1224,92 @@ def abortOnSignal(signum, frame):
 
 def usage():
     """Prints out usage information."""
-    print "Usage: orca [OPTION...]"
+    print _("Usage: orca [OPTION...]")
     print
-    print "-?, --help                   Show this help message"
+
+    # Translators: this is the description of the command line option 
+    # '-?, --help' that is used to display usage information.
+    #
+    print "-?, --help                   " + _("Show this help message")
+
     print "-v, --version                %s" % platform.version
-    print "-s, --setup, --gui-setup     Set up user preferences"
-    print "-t, --text-setup             Set up user preferences (text version)"
-    print "-n, --no-setup               Skip set up of user preferences"
-    print "-u, --user-prefs-dir=dirname Use alternate directory for user preferences"
-    print "-e, --enable=[speech|braille|braille-monitor|magnifier|main-window] Force use of option"
-    print "-d, --disable=[speech|braille|braille-monitor|magnifier|main-window] Prevent use of option"
-    print "-q, --quit                   Quits Orca (if shell script used)"
+
+    # Translators: this is the description of the command line option
+    # '-s, --setup, --gui-setup' that will initially display a GUI dialog 
+    # that would allow the user to set their Orca preferences.
+    #
+    print "-s, --setup, --gui-setup     " + _("Set up user preferences")
+
+    # Translators: this is the description of the command line option 
+    # '-t, --text-setup' that will initially display a list of questions 
+    # in text form, that the user will need to answer, before Orca will 
+    # startup. For this to happen properly, Orca will need to be run 
+    # from a terminal window.
+    #
+    print "-t, --text-setup             " + \
+          _("Set up user preferences (text version)")
+
+    # Translators: this is the description of the command line option 
+    # '-n, --no-setup' that means that Orca will startup without setting 
+    # up any user preferences.
+    #
+    print "-n, --no-setup               " +  \
+          _("Skip set up of user preferences")
+
+    # Translators: by default, Orca expects to find its user preferences
+    # in a directory called .orca under the user's home directory. This
+    # is the description of the command line option 
+    # '-u, --user-prefs-dir=dirname' that allows you to specify an alternate
+    # location for those user preferences.
+    #
+    print "-u, --user-prefs-dir=dirname " + \
+          _("Use alternate directory for user preferences")
+
+    print "-e, --enable=[" \
+        + Q_("option|speech") + "|" \
+        + Q_("option|braille") + "|" \
+        + Q_("option|braille-monitor") + "|" \
+        + Q_("option|magnifier") + "|" \
+        + Q_("option|main-window") + "]",
+
+    # Translators: if the user supplies an option via the '-e, --enable'
+    # command line option, it will be automatically enabled as Orca is
+    # started.
+    #
+    print _("Force use of option")
+
+    print "-d, --disable=[" \
+        + Q_("option|speech") + "|" \
+        + Q_("option|braille") + "|" \
+        + Q_("option|braille-monitor") + "|" \
+        + Q_("option|magnifier") + "|" \
+        + Q_("option|main-window") + "]",
+
+    # Translators: if the user supplies an option via the '-d, --disable'
+    # command line option, it will be automatically disabled as Orca is
+    # started.
+    #
+    print _("Prevent use of option") 
+
+    # Translators: this is the Orca command line option that will quit Orca.
+    # The user would run the Orca shell script again from a terminal window.
+    # If this command line option is specified, the script will quit any
+    # instances of Orca that are already running.
+    #
+    print "-q, --quit                   " + \
+          _("Quits Orca (if shell script used)")
+
     print
-    print "If Orca has not been previously set up by the user, Orca\nwill automatically launch the preferences set up unless\nthe -n or --no-setup option is used."
+
+    # Translators: this is text being sent to a terminal and we want to 
+    # keep the text lines within terminal boundaries.
+    #
+    print _("If Orca has not been previously set up by the user, Orca\n" \
+            "will automatically launch the preferences set up unless\n" \
+            "the -n or --no-setup option is used.")
+
     print
-    print "Report bugs to orca-list@gnome.org."
-    pass
+    print _("Report bugs to orca-list@gnome.org.")
 
 def main():
     """The main entry point for Orca.  The exit codes for Orca will
@@ -1310,8 +1318,6 @@ def main():
     an exit code of 0 means normal completion and an exit code of 50
     means Orca exited because of a hang."""
 
-    global _commandLineSettings
-
     # Method to call when we think something might be hung.
     #
     settings.timeoutCallback = timeout
@@ -1379,7 +1385,7 @@ def main():
              "version"])
         for opt, val in opts:
             if opt in ("-u", "--user-prefs-dir"):
-                userPrefsDir = val.strip();
+                userPrefsDir = val.strip()
                 try:
                     os.chdir(userPrefsDir)
                     settings.userPrefsDir = userPrefsDir
@@ -1388,31 +1394,62 @@ def main():
 
             if opt in ("-e", "--enable"):
                 feature = val.strip()
-                if feature == "speech":
+
+                # Translators: this option is for enabling speech synthesis
+                # output.
+                #
+                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+                #
+                if feature == Q_("option|speech"):
                     _commandLineSettings["enableSpeech"] = True
-                elif feature == "braille":
+
+                # Translators: this option is for enabling braille output on
+                # a physical refreshable braille display.
+                #
+                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+                #
+                elif feature == Q_("option|braille"):
                     _commandLineSettings["enableBraille"] = True
-                elif feature == "braille-monitor":
+
+                # Translators: this option is for enabling a GUI to monitor 
+                # what is being sent to the physical braille display.
+                #
+                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+                #
+                elif feature == Q_("option|braille-monitor"):
                     _commandLineSettings["enableBrailleMonitor"] = True
-                elif feature == "magnifier":
+
+                # Translators: this option is for enabling screen 
+                # magnification.
+                #
+                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+                #
+                elif feature == Q_("option|magnifier"):
                     _commandLineSettings["enableMagnifier"] = True
-                elif feature == "main-window":
+
+                # Translators: this option is for enabling the display
+                # of the Orca main window.
+                #
+                # ONLY TRANSLATE THE PART AFTER THE PIPE CHARACTER |
+                #
+                elif feature == Q_("option|main-window"):
                     _commandLineSettings["showMainWindow"] = True
+
                 else:
                     usage()
                     os._exit(2)
 
             if opt in ("-d", "--disable"):
                 feature = val.strip()
-                if feature == "speech":
+                if feature == Q_("option|speech"):
                     _commandLineSettings["enableSpeech"] = False
-                elif feature == "braille":
+                elif feature == Q_("option|braille"):
                     _commandLineSettings["enableBraille"] = False
-                elif feature == "braille-monitor":
+                elif feature == Q_("option|braille-monitor"):
                     _commandLineSettings["enableBrailleMonitor"] = False
-                elif feature == "magnifier":
+                elif feature == Q_("option|magnifier"):
                     _commandLineSettings["enableMagnifier"] = False
-                elif feature == "main-window":
+                elif feature == Q_("option|main-window"):
                     _commandLineSettings["showMainWindow"] = False
                 else:
                     usage()
@@ -1462,8 +1499,7 @@ def main():
     sys.path.insert(0, userprefs)
     sys.path.insert(0, '') # current directory
 
-    registry = atspi.Registry()
-    init(registry)
+    init(pyatspi.Registry)
 
     try:
         message = _("Welcome to Orca.")
@@ -1486,7 +1522,7 @@ def main():
         else:
             _showPreferencesConsole()
 
-    start(registry) # waits until we stop the registry
+    start(pyatspi.Registry) # waits until we stop the registry
     return 0
 
 if __name__ == "__main__":
