@@ -19,9 +19,9 @@
 
 """Custom script for Evolution."""
 
-__id__        = "$Id: Evolution.py 3187 2007-10-26 04:21:32Z joanied $"
-__version__   = "$Revision: 3187 $"
-__date__      = "$Date: 2007-10-26 00:21:32 -0400 (Fri, 26 Oct 2007) $"
+__id__        = "$Id: Evolution.py 3230 2007-11-09 17:02:46Z richb $"
+__version__   = "$Revision: 3230 $"
+__date__      = "$Date: 2007-11-09 12:02:46 -0500 (Fri, 09 Nov 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -700,7 +700,7 @@ class Script(default.Script):
 
         debug.printObjectEvent(self.debugLevel,
                                event,
-                               event.source)
+                               debug.getAccessibleDetails(event.source))
 
         # self.printAncestry(event.source)
 
